'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

exports.customLocalLambdaAuthorizer = customLocalLambdaAuthorizer;

var _lodash = require('lodash');

var _authFunction = require('./auth-function');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var debug = require('debug')('lambda-custom-authorizer-middleware');
var localAuthorizer = void 0;

/**
 * Express middleware function constructor to execute local lambda function
 * as a custom authorizer and attach request context to `req` object
 * as `req.apiGateway.event.requestContext.authorizer` (as for usage with `aws-serverless-exporess` npm package)
 * @param {Object} options Configuration object
 * @param {String} [options.identitySourceHeader=authorization] Name of HTTP header where auth token is located
 * @param {Object} [options.localAuthorizer={}] Local authorizer function configuration object
 * @param {String} options.localAuthorizer.handlerPath Path on local file system to the function
 * @param {String} options.localAuthorizer.handlerName Name of the exported function in provided path
 * @return {Function} Express middleware function. Works only when IS_OFFLINE env var is set.
 * @throws {Error} Throws when config is not provided
 * @example
 *
 * import express from 'express';
 * import awsSlsExpressMiddleware from 'aws-serverless-express/middleware';
 * import {customLocalLambdaAuthorizer} from 'lambda-custom-authorizer-middleware';
 *
 * const app = express();
 *
 * app.use(awsSlsExpressMiddleware.eventContext());
 * app.use(customLocalLambdaAuthorizer({ // Make sure to add after 'awsSlsExpressMiddleware'
 *  localAuthorizer: {
 *    handlerPath: '../other-project/lambda/auth',
 *    handlerName: 'handler'
 *  }
 * }));
 *
 * app.get('/', (req, res) => res.json(req.apiGateway.event.requestContext.authorizer));
 */
function customLocalLambdaAuthorizer() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$identitySourceHe = _ref.identitySourceHeader,
      identitySourceHeader = _ref$identitySourceHe === undefined ? 'authorization' : _ref$identitySourceHe,
      _ref$localAuthorizer = _ref.localAuthorizer;

  _ref$localAuthorizer = _ref$localAuthorizer === undefined ? {} : _ref$localAuthorizer;
  var handlerPath = _ref$localAuthorizer.handlerPath,
      handlerName = _ref$localAuthorizer.handlerName;

  if (!handlerPath || !handlerName) {
    throw new Error('Please provide config for customLocalLambdaAuthorizer!');
  }

  return function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee(req, res, next) {
      var isOffline, identitySourceValue, _ref3, context;

      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              isOffline = Boolean(process.env.IS_OFFLINE);

              debug(`[is-offline:${isOffline}]`);

              if (isOffline) {
                _context.next = 5;
                break;
              }

              debug(`[skip]`);
              return _context.abrupt('return', next());

            case 5:

              localAuthorizer = (0, _authFunction.getAuthFunction)(handlerPath, handlerName);

              _context.prev = 6;
              identitySourceValue = (0, _lodash.get)(req, `headers[${(0, _lodash.camelCase)(identitySourceHeader)}]`, '');


              if (!identitySourceValue) {
                debug('[warn][empty-header-value]');
              }

              _context.next = 11;
              return localAuthorizer({
                authorizationToken: identitySourceValue,
                methodArn: 'lambda-custom-authorizer-middleware'
              });

            case 11:
              _ref3 = _context.sent;
              context = _ref3.context;

              debug(`[success] %o`, context);

              (0, _lodash.set)(req, 'apiGateway.event.requestContext.authorizer', context);
              debug(`[context-set:apiGateway.event.requestContext.authorizer]`);
              _context.next = 22;
              break;

            case 18:
              _context.prev = 18;
              _context.t0 = _context['catch'](6);

              debug(`[error:${_context.t0.message}] %o`, _context.t0);
              return _context.abrupt('return', res.status(401).json({ error: _context.t0 }));

            case 22:

              debug(`[done]`);
              return _context.abrupt('return', next());

            case 24:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this, [[6, 18]]);
    }));

    return function (_x2, _x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }();
}